/*
 *  CPAchecker is a tool for configurable software verification.
 *  This file is part of CPAchecker.
 *
 *  Copyright (C) 2007-2010  Dirk Beyer
 *  All rights reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *
 *  CPAchecker web page:
 *    http://cpachecker.sosy-lab.org
 */
package org.sosy_lab.cpachecker.cfa.ast;

import java.math.BigInteger;

public class IASTIntegerLiteralExpression extends IASTLiteralExpression {

  // use BigInteger here because a C unsigned long long constant doesn't fit in
  // a Java long
  private final BigInteger value;

  public IASTIntegerLiteralExpression(String pRawSignature,
      IASTFileLocation pFileLocation, IType pType, BigInteger pValue) {
    super(pRawSignature, pFileLocation, pType, IASTLiteralExpression.lk_integer_constant);
    value = pValue;
  }
  
  public BigInteger getValue() {
    return value;
  }
  
  @Override
  public <R, X extends Exception> R accept(ExpressionVisitor<R, X> v) throws X {
    return v.visit(this);
  }
  
  @Override
  public <R, X extends Exception> R accept(RightHandSideVisitor<R, X> v) throws X {
    return v.visit(this);
  }
}
