/*
 *  CPAchecker is a tool for configurable software verification.
 *  This file is part of CPAchecker.
 *
 *  Copyright (C) 2007-2010  Dirk Beyer
 *  All rights reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *
 *  CPAchecker web page:
 *    http://cpachecker.sosy-lab.org
 */
package org.sosy_lab.cpachecker.cpa.predicate;

import java.util.Collection;
import java.util.Set;

import org.sosy_lab.cpachecker.cfa.objectmodel.CFANode;
import org.sosy_lab.cpachecker.core.interfaces.Precision;
import org.sosy_lab.cpachecker.util.predicates.AbstractionPredicate;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.ImmutableSetMultimap;
import com.google.common.collect.SetMultimap;

public class PredicatePrecision implements Precision {

  private final ImmutableSetMultimap<CFANode, AbstractionPredicate> predicateMap;
  private final ImmutableSet<AbstractionPredicate> globalPredicates;

  public PredicatePrecision(ImmutableSetMultimap<CFANode, AbstractionPredicate> predicateMap, Collection<AbstractionPredicate> globalPredicates) {
    assert predicateMap != null;
    this.predicateMap = predicateMap;
    this.globalPredicates = ImmutableSet.copyOf(globalPredicates);
  }

  public PredicatePrecision(Collection<AbstractionPredicate> globalPredicates) {
    predicateMap = ImmutableSetMultimap.of();
    this.globalPredicates = (globalPredicates == null ? ImmutableSet.<AbstractionPredicate>of() : ImmutableSet.copyOf(globalPredicates));
  }

  public SetMultimap<CFANode, AbstractionPredicate> getPredicateMap() {
    return predicateMap;
  }

  public Set<AbstractionPredicate> getGlobalPredicates() {
    return globalPredicates;
  }
  
  public Set<AbstractionPredicate> getPredicates(CFANode loc) {
    Set<AbstractionPredicate> result = predicateMap.get(loc);
    if (result.isEmpty()) {
      result = globalPredicates;
    }
    return result;
  }

  @Override
  public int hashCode() {
    return predicateMap.hashCode();
  }

  @Override
  public boolean equals(Object pObj) {
    if (pObj == this) {
      return true;
    } else if (!(pObj instanceof PredicatePrecision)) {
      return false;
    } else {
      return predicateMap.equals(((PredicatePrecision)pObj).predicateMap);
    }
  }

  @Override
  public String toString() {
    return predicateMap.toString();
  }
}


